"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AutoJsDebugServer = exports.Device = void 0;
const tslib_1 = require("tslib");
const events_1 = require("events");
const websocket_1 = tslib_1.__importDefault(require("websocket"));
const http_1 = tslib_1.__importDefault(require("http"));
const querystring_1 = tslib_1.__importDefault(require("querystring"));
const url_1 = tslib_1.__importDefault(require("url"));
const adbkit_1 = tslib_1.__importDefault(require("@devicefarmer/adbkit"));
const project_1 = require("./project");
const DEBUG = false;
const HANDSHAKE_TIMEOUT = 10 * 1000;
class Device extends events_1.EventEmitter {
    constructor(connection, type, id) {
        super();
        Object.defineProperty(this, "name", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "type", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "id", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "connection", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "attached", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        Object.defineProperty(this, "projectObserser", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "toString", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: () => {
                if (!this.name) {
                    return `设备 (${this.type}: ${this.id})`;
                }
                return `设备 ${this.name}(${this.type}: ${this.id})`;
            }
        });
        this.type = type;
        this.id = id;
        this.connection = connection;
        this.read(this.connection);
        this.on('data:hello', data => {
            var _a;
            console.log("on client hello: ", data);
            this.attached = true;
            this.name = data['device_name'];
            let message_id = `${Date.now()}_${Math.random()}`;
            let appVersionCode = data['app_version_code'];
            let returnData, version = "1.109.0"; // 扩展版本
            if (appVersionCode >= 629) {
                returnData = { message_id, data: "ok", version, debug: DEBUG, type: 'hello' };
            }
            else {
                returnData = { message_id, data: "连接成功", debug: DEBUG, type: 'hello' };
            }
            console.log("return data: ", returnData);
            (_a = this.connection) === null || _a === void 0 ? void 0 : _a.sendUTF(JSON.stringify(returnData));
            this.emit("attach", this);
        });
        this.on('data:ping', data => {
            var _a;
            console.log("on client ping: ", data);
            // let message_id = `${Date.now()}_${Math.random()}`;
            const returnData = JSON.stringify({ type: 'pong', data: data });
            console.log("pong: ", returnData);
            (_a = this.connection) === null || _a === void 0 ? void 0 : _a.sendUTF(returnData);
        });
        setTimeout(() => {
            var _a;
            if (!this.attached) {
                console.log("handshake timeout");
                (_a = this.connection) === null || _a === void 0 ? void 0 : _a.close();
                this.connection = null;
            }
        }, HANDSHAKE_TIMEOUT);
    }
    close() {
        var _a, _b;
        let message_id = `${Date.now()}_${Math.random()}`;
        let closeMessage = JSON.stringify({ message_id, data: "close", debug: false, type: 'close' });
        (_a = this.connection) === null || _a === void 0 ? void 0 : _a.sendUTF(closeMessage);
        (_b = this.connection) === null || _b === void 0 ? void 0 : _b.close();
        this.connection = null;
    }
    send(type, data) {
        var _a;
        let message_id = `${Date.now()}_${Math.random()}`;
        console.log(data);
        (_a = this.connection) === null || _a === void 0 ? void 0 : _a.sendUTF(JSON.stringify({
            type: type,
            message_id,
            data: data
        }));
    }
    sendBytes(bytes) {
        var _a;
        (_a = this.connection) === null || _a === void 0 ? void 0 : _a.sendBytes(bytes);
    }
    sendBytesCommand(command, md5, data = {}) {
        var _a;
        data = Object(data);
        data['command'] = command;
        let message_id = `${Date.now()}_${Math.random()}`;
        (_a = this.connection) === null || _a === void 0 ? void 0 : _a.sendUTF(JSON.stringify({
            type: 'bytes_command',
            message_id,
            md5: md5,
            data: data
        }));
    }
    sendCommand(command, data) {
        data = Object(data);
        data['command'] = command;
        this.send('command', data);
    }
    read(connection) {
        connection.on('message', message => {
            console.log("消息到达: ", message);
            if (message.type == 'utf8') {
                try {
                    let json = JSON.parse(message.utf8Data);
                    console.log("json: ", json);
                    this.emit('message', json);
                    this.emit('data:' + json['type'], json['data']);
                }
                catch (e) {
                    console.log(e);
                }
            }
        });
        connection.on('close', (reasonCode, description) => {
            console.log(`关闭: 设备 = ${this}, 原因 = ${reasonCode}, 描述 = ${description}`);
            this.connection = null;
            this.emit('disconnect');
        });
    }
}
exports.Device = Device;
class AutoJsDebugServer extends events_1.EventEmitter {
    constructor(port = 9317) {
        super();
        Object.defineProperty(this, "isHttpServerStarted", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        Object.defineProperty(this, "httpServer", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "adbClient", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: adbkit_1.default.createClient()
        });
        Object.defineProperty(this, "tracker", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "port", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "devices", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        Object.defineProperty(this, "project", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: null
        });
        Object.defineProperty(this, "fileFilter", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (relativePath, absPath, stats) => {
                if (!this.project) {
                    return true;
                }
                return this.project.fileFilter(relativePath, absPath, stats);
            }
        });
        this.port = port;
        this.httpServer = http_1.default.createServer((request, response) => {
            console.log(new Date() + ' Received request for ' + request.url);
            if (request.url) {
                const urlObj = url_1.default.parse(request.url);
                const query = urlObj.query;
                if (query) {
                    const queryObj = querystring_1.default.parse(query);
                    if (urlObj.pathname == "/exec") {
                        response.writeHead(200);
                        response.end("this commond is:" + queryObj.cmd + "-->" + queryObj.path);
                        this.emit('cmd', queryObj.cmd, queryObj.path);
                        console.log(queryObj.cmd, queryObj.path);
                        return;
                    }
                }
            }
            response.writeHead(404);
            response.end();
        });
        let wsServer = new websocket_1.default.server({ httpServer: this.httpServer, keepalive: true, keepaliveInterval: 10000 });
        wsServer.on('request', request => {
            let connection = request.accept();
            if (!connection) {
                return;
            }
            this.newDevice(connection, "tcp", connection.socket.remoteAddress + ":" + connection.socket.remotePort);
        });
    }
    getDeviceById(id) {
        return this.devices.find((value) => {
            return value.id == id;
        });
    }
    newDevice(connection, type, id) {
        let device = new Device(connection, type, id);
        console.log("新设备进入", id, type), connection;
        device
            .on("attach", (device) => {
            this.attachDevice(device);
            this.emit('new_device', device);
            console.log("设备连接:", device);
        }).on("message", (data) => {
            this.emit("message", data, device);
        });
    }
    adbShell(device, command) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let duplex = yield device.shell(command);
            let brandBuf = yield adbkit_1.default.util.readAll(duplex);
            return brandBuf.toString();
        });
    }
    connectAutoxjsByADB(port, deviceId) {
        let autoJsDebugServer = this;
        let url = `ws://localhost:${port}/`;
        const client = new websocket_1.default.client();
        client.on("connectFailed", function (error) {
            let err = '连接错误: ' + error.toString();
            console.log(err);
        });
        client.on("connect", function (connection) {
            console.log("connected to " + url);
            autoJsDebugServer.newDevice(connection, "adb", deviceId);
        });
        client.connect(url);
    }
    listen() {
        if (this.isHttpServerStarted) {
            this.emit("connected");
            return;
        }
        this.httpServer.on('error', (e) => {
            this.isHttpServerStarted = false;
            console.log('server error: ', e);
        });
        this.httpServer.listen(this.port, '0.0.0.0', () => {
            this.isHttpServerStarted = true;
            const address = this.httpServer.address();
            const localAddress = this.getIPAddress();
            // console.log(`服务开始监听: ${localAddress}:${address.port} / ${address.address}:${address.port}`);
            this.emit("connect");
        });
    }
    listADBDevices() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.adbClient.listDevices();
        });
    }
    trackADBDevices() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let thisServer = this;
            let devices = yield thisServer.adbClient.listDevices();
            for (let device0 of devices) {
                yield thisServer.connectDevice(device0.id);
            }
            if (this.tracker) {
                this.emit("adb:tracking_started");
                return;
            }
            try {
                let tracker = yield thisServer.adbClient.trackDevices();
                thisServer.tracker = tracker;
                tracker.on('add', function (device0) {
                    return tslib_1.__awaiter(this, void 0, void 0, function* () {
                        console.log("adb device " + device0.id + " added");
                        const device = thisServer.adbClient.getDevice(device0.id);
                        yield device.waitForDevice();
                        yield thisServer.connectDevice(device0.id, device);
                    });
                });
                tracker.on('remove', function (device) {
                    console.log("adb device " + device.id + " removed");
                    let wsDevice = thisServer.getDeviceById(device.id);
                    if (wsDevice) {
                        wsDevice.close();
                    }
                });
                tracker.on('end', function () {
                    thisServer.tracker = undefined;
                    console.log('ADB Tracking stopped');
                    thisServer.emit("adb:tracking_stop");
                });
            }
            catch (err) {
                this.tracker = undefined;
                thisServer.emit("adb:tracking_error");
                console.log('ADB error: ', err.stack);
            }
            this.emit("adb:tracking_start");
        });
    }
    connectDevice(id, device) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!device)
                device = this.adbClient.getDevice(id);
            let wsDevice = this.getDeviceById(id);
            if (wsDevice && wsDevice.attached)
                return;
            let forwards = yield this.listForwards(device, id);
            if (forwards.length == 0) {
                let forwarded = yield device.forward(`tcp:0`, `tcp:9317`);
                if (forwarded) {
                    forwards = yield this.listForwards(device, id);
                }
            }
            if (forwards.length > 0) {
                let forward = forwards[0];
                console.log(`forward ${id}: local -> ${forward.local}, remote -> ${forward.remote}`);
                let port = Number(forward.local.replace("tcp:", ""));
                this.connectAutoxjsByADB(port, id);
            }
        });
    }
    listForwards(device, id) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let forwards = yield device.listForwards();
            return forwards.filter((forward) => {
                return forward.serial == id && forward.remote == "tcp:9317";
            });
        });
    }
    stopTrackADBDevices() {
        if (this.tracker) {
            this.tracker.end();
            this.tracker = undefined;
        }
    }
    send(type, data) {
        this.devices.forEach(device => {
            device.send(type, data);
        });
    }
    sendBytes(data) {
        this.devices.forEach(device => {
            device.sendBytes(data);
        });
    }
    sendBytesCommand(command, md5, data = {}) {
        this.devices.forEach(device => {
            device.sendBytesCommand(command, md5, data);
        });
    }
    // 保存项目 sendProjectCommand( save_project, 项目路径)
    // 运行项目 sendProjectCommand( run_project)
    sendProjectCommand(folder, command) {
        // const startTime = new Date().getTime();
        this.devices.forEach(device => {
            if (device.projectObserser == null || device.projectObserser.folder != folder) {
                device.projectObserser = new project_1.ProjectObserser(folder, this.fileFilter);
            }
            device.projectObserser.diff()
                .then(result => {
                var _a;
                device.sendBytes(result.buffer);
                device.sendBytesCommand(command, result.md5, {
                    'id': device.id,
                    'name': (_a = this.project) === null || _a === void 0 ? void 0 : _a.config.name
                });
                // console.log(`发送项目耗时: ${(new Date().getTime() - startTime) / 1000} 秒`);
            });
        });
    }
    sendCommand(command, data = {}) {
        this.devices.forEach(device => {
            device.sendCommand(command, data);
        });
    }
    disconnect() {
        this.httpServer.close();
        this.isHttpServerStarted = false;
        this.emit("disconnect");
        this.devices.forEach((device) => {
            device.close();
        });
    }
    /** 获取本地IP */
    getIPAddress() {
        const interfaces = require('os').networkInterfaces();
        for (let devName in interfaces) {
            const iface = interfaces[devName];
            for (let i = 0; i < iface.length; i++) {
                const alias = iface[i];
                if (alias.family === 'IPv4' && alias.address !== '127.0.0.1' && !alias.internal) {
                    // console.log("本机ip信息:", alias);
                    return alias.address;
                }
            }
        }
        console.log("没有获取到本机IP信息");
    }
    /** 获取本地IP */
    getIPs() {
        const ips = [];
        const interfaces = require('os').networkInterfaces();
        for (let devName in interfaces) {
            const iface = interfaces[devName];
            for (let i = 0; i < iface.length; i++) {
                const alias = iface[i];
                console.log("---", alias);
                if (alias.family === 'IPv4' && alias.address !== '127.0.0.1' && !alias.internal) {
                    ips.push(alias.address);
                }
            }
        }
        return ips;
    }
    /** 获取服务运行端口 */
    getPort() {
        return this.port;
    }
    attachDevice(device) {
        this.devices.push(device);
        device.on('data:log', data => {
            console.log(data['log']);
            this.emit('log', data['log']);
        });
        device.on('disconnect', this.detachDevice.bind(this, device));
    }
    /** 分离设备? */
    detachDevice(device) {
        this.devices.splice(this.devices.indexOf(device), 1);
        console.log("detachDevice: " + device);
    }
}
exports.AutoJsDebugServer = AutoJsDebugServer;
