"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProjectConfig = exports.LaunchConfig = exports.ProjectObserser = exports.Project = void 0;
const tslib_1 = require("tslib");
const diff_1 = require("./diff");
const archiver_1 = tslib_1.__importDefault(require("archiver"));
const streamBuffers = tslib_1.__importStar(require("stream-buffers"));
const fs = tslib_1.__importStar(require("fs"));
const path = tslib_1.__importStar(require("path"));
const cryto = tslib_1.__importStar(require("crypto"));
const walk = tslib_1.__importStar(require("walk"));
class Project {
    // private watcher: vscode.FileSystemWatcher;
    constructor(configPath, config) {
        Object.defineProperty(this, "config", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "fpath", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "fileFilter", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (relativePath, absPath, stats) => {
                return this.config.ignore.filter(p => {
                    const fullPath = path.join(this.fpath, p);
                    return absPath.startsWith(fullPath);
                }).length == 0;
            }
        });
        this.fpath = configPath;
        this.config = ProjectConfig.fromJsonFile(config);
        // console.error("创建了Project对象, 需要监听项目文件变化");
    }
    dispose() {
        // this.watcher.dispose();
        console.error("执行了处理");
    }
}
exports.Project = Project;
class ProjectObserser {
    constructor(folder, filter) {
        Object.defineProperty(this, "folder", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "fileObserver", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "fileFilter", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.folder = folder;
        this.fileFilter = filter;
        this.fileObserver = new diff_1.FileObserver(folder, filter);
    }
    diff() {
        return this.fileObserver.walk()
            .then(changedFiles => {
            const zip = (0, archiver_1.default)('zip');
            const streamBuffer = new streamBuffers.WritableStreamBuffer();
            zip.pipe(streamBuffer);
            changedFiles.forEach(relativePath => {
                zip.append(fs.createReadStream(path.join(this.folder, relativePath)), { name: relativePath });
            });
            zip.finalize();
            return new Promise((res, rej) => {
                zip.on('finish', () => {
                    streamBuffer.end();
                    res(streamBuffer.getContents());
                });
            });
        })
            .then(buffer => {
            const md5 = cryto.createHash('md5').update(buffer).digest('hex');
            return {
                buffer: buffer,
                md5: md5
            };
        });
    }
    zip() {
        return new Promise((res, rej) => {
            const walker = walk.walk(this.folder);
            const zip = (0, archiver_1.default)('zip');
            const streamBuffer = new streamBuffers.WritableStreamBuffer();
            zip.pipe(streamBuffer);
            walker.on("file", (root, stat, next) => {
                const filePath = path.join(root, stat.name);
                const relativePath = path.relative(this.folder, filePath);
                if (!this.fileFilter(relativePath, filePath, stat)) {
                    next();
                    return;
                }
                zip.append(fs.createReadStream(path.join(this.folder, relativePath)), { name: relativePath });
                next();
            });
            walker.on("end", () => {
                zip.finalize();
                return new Promise((res, rej) => {
                    zip.on('finish', () => {
                        streamBuffer.end();
                        res(streamBuffer.getContents());
                    });
                });
            });
        });
    }
}
exports.ProjectObserser = ProjectObserser;
class LaunchConfig {
    constructor() {
        Object.defineProperty(this, "hideLogs", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
    }
}
exports.LaunchConfig = LaunchConfig;
class ProjectConfig {
    constructor() {
        Object.defineProperty(this, "name", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "icon", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "packageName", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "main", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "versionCode", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "versionName", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "ignore", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "launchConfig", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
    }
    save(path) {
        return new Promise((res, rej) => {
            const json = JSON.stringify(this, null, 4);
            fs.writeFile(path, json, function (err) {
                if (err) {
                    rej(err);
                    return;
                }
                res(path);
            });
        });
    }
    static fromJson(text) {
        const config = JSON.parse(text);
        config.ignore = (config.ignore || []).map(p => path.normalize(p));
        return config;
    }
    static fromJsonFile(config) {
        config.ignore = (config.ignore || []);
        return config;
    }
}
exports.ProjectConfig = ProjectConfig;
